@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2013 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
  *
  * Authors:
  *      Maros Barabas <mbarabas@redhat.com>
+ *      Šimon Lukašík
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,15 +38,21 @@
 #include <ctype.h>
 // for functins memset, strcpy
 #include <string.h>
+#include <stdbool.h>
 
 #include "public/cpe_dict.h"
 #include "public/cpe_name.h"
+#include "cpedict_ext_priv.h"
 #include "cpedict_priv.h"
+#include "cpe_ctx_priv.h"
 
 #include "common/list.h"
 #include "common/elements.h"
 #include "common/text_priv.h"
+#include "common/util.h"
 #include "common/_error.h"
+#include "common/xmlns_priv.h"
+#include "common/xmltext_priv.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -64,16 +71,27 @@ struct cpe_item {		// the node <cpe-item>
 	struct cpe_name *name;	// CPE name as CPE URI
 	struct oscap_list *titles;	// titles of cpe-item (can be in various languages)
 
-	struct cpe_name *deprecated;	// CPE that deprecated this one (or NULL)
+	bool deprecated;			///< deprecated attribute
+	struct cpe_name *deprecated_by;		///< deprecated_by attribute, the CPE name that deprecated this
 	char *deprecation_date;	// date of deprecation
 
 	struct oscap_list *references;	// list of references
 	struct oscap_list *checks;	// list of checks
 	struct oscap_list *notes;	// list of notes - it's the same structure as titles
 	struct cpe_item_metadata *metadata;	// element <meta:item-metadata>
+	struct cpe23_item *cpe23_item;		///< element <cpe23-item>
+	struct {
+		bool deprecated:1;		///< Is the deprecated atrtribute specified in XML?
+	} export;
 };
 OSCAP_GETTER(struct cpe_name *, cpe_item, name)
-OSCAP_GETTER(struct cpe_name *, cpe_item, deprecated)
+OSCAP_GETTER(struct cpe_name *, cpe_item, deprecated_by)
+OSCAP_DEPRECATED(
+	struct cpe_name *cpe_item_get_deprecated(const struct cpe_item *item)
+	{
+		return cpe_item_get_deprecated_by(item);
+	}
+)
 OSCAP_ACCESSOR_STRING(cpe_item, deprecation_date)
 OSCAP_GETTER(struct cpe_item_metadata *, cpe_item, metadata)
 OSCAP_IGETINS_GEN(cpe_reference, cpe_item, references, reference)
@@ -82,6 +100,11 @@ OSCAP_IGETINS(oscap_text, cpe_item, titles, title)
 OSCAP_IGETINS(oscap_text, cpe_item, notes, note)
 OSCAP_ITERATOR_REMOVE_F(cpe_reference) OSCAP_ITERATOR_REMOVE_F(cpe_check)
 
+struct cpe_notes {				///< representation of <notes> element
+	char *lang;				///< xml:lang attribute
+	struct oscap_list *notes;		///< list of inner <note> elements
+};
+
 /* <cpe-item><item-metadata>
  * */
 struct cpe_item_metadata {
@@ -241,10 +264,6 @@ OSCAP_ACCESSOR_STRING(cpe_language, value)
 #define NS_META_STR         BAD_CAST "meta"
 #define ATTR_XML_LANG_STR   BAD_CAST "xml:lang"
 #define VAL_TRUE_STR        BAD_CAST "true"
-/* Namespaces */
-#define CPE_1_DICT_NS BAD_CAST "http://cpe.mitre.org/XMLSchema/cpe/1.0"
-#define CPE_2_DICT_NS BAD_CAST "http://cpe.mitre.org/dictionary/2.0"
-#define CPEMETA_NS BAD_CAST "http://scap.nist.gov/schema/cpe-dictionary-metadata/0.2"
 /* End of XML string variables definitions
  * */
 /***************************************************************************/
@@ -253,11 +272,11 @@ OSCAP_ACCESSOR_STRING(cpe_language, value)
  * These function shoud not be called from outside. For exporting these elements
  * has to call parent element's 
  */
-static int xmlTextReaderNextElement(xmlTextReaderPtr reader);
 static bool cpe_dict_model_add_item(struct cpe_dict_model *dict, struct cpe_item *item);
 
 static struct cpe_reference *cpe_reference_parse(xmlTextReaderPtr reader);
 static struct cpe_check *cpe_check_parse(xmlTextReaderPtr reader);
+static struct cpe_notes *cpe_notes_parse(xmlTextReaderPtr reader);
 
 static void cpe_product_export(const struct cpe_product *product, xmlTextWriterPtr writer);
 static void cpe_version_export(const struct cpe_version *version, xmlTextWriterPtr writer);
@@ -266,9 +285,11 @@ static void cpe_edition_export(const struct cpe_edition *edition, xmlTextWriterP
 static void cpe_language_export(const struct cpe_language *language, xmlTextWriterPtr writer);
 static void cpe_check_export(const struct cpe_check *check, xmlTextWriterPtr writer);
 static void cpe_reference_export(const struct cpe_reference *ref, xmlTextWriterPtr writer);
+static void cpe_notes_export(const struct cpe_notes *notes, xmlTextWriterPtr writer);
 
 static bool cpe_validate_xml(const char *filename);
-static int xmlTextReaderNextNode(xmlTextReaderPtr reader);
+struct cpe_notes *cpe_notes_new(void);
+void cpe_notes_free(struct cpe_notes *notes);
 /***************************************************************************/
 
 /* Add item to dictionary. Function that just check both variables
@@ -287,78 +308,6 @@ static bool cpe_dict_model_add_item(struct cpe_dict_model *dict, struct cpe_item
 	return true;
 }
 
-/* Function that jump to next XML starting element.
- * */
-static int xmlTextReaderNextElement(xmlTextReaderPtr reader)
-{
-
-	__attribute__nonnull__(reader);
-
-	int ret;
-	do {
-		ret = xmlTextReaderRead(reader);
-		// if end of file
-		if (ret < 1)
-			break;
-	} while (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
-
-	if (ret == -1) {
-		oscap_setxmlerr(xmlCtxtGetLastError(reader));
-		/* TODO: Should we end here as fatal ? */
-	}
-
-	return ret;
-}
-
-/* Function that jump to next XML starting element.
- *
- * This function makes sure we don't go past end tag of given element
- * */
-static int xmlTextReaderNextElementWE(xmlTextReaderPtr reader, xmlChar* end_tag)
-{
-
-	__attribute__nonnull__(reader);
-
-	int ret;
-	do {
-		ret = xmlTextReaderRead(reader);
-		// if end of file
-		if (ret < 1)
-			break;
-
-		if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT) {
-			if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), end_tag)) {
-				ret = 0;
-				break;
-			}
-		}
-	} while (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
-
-	if (ret == -1) {
-		oscap_setxmlerr(xmlCtxtGetLastError(reader));
-		/* TODO: Should we end here as fatal ? */
-	}
-
-	return ret;
-}
-
-/* Function testing reader function 
- */
-static int xmlTextReaderNextNode(xmlTextReaderPtr reader)
-{
-
-	__attribute__nonnull__(reader);
-
-	int ret;
-	ret = xmlTextReaderRead(reader);
-	if (ret == -1) {
-		oscap_setxmlerr(xmlCtxtGetLastError(reader));
-		/* TODO: Should we end here as fatal ? */
-	}
-
-	return ret;
-}
-
 static bool cpe_validate_xml(const char *filename)
 {
 
@@ -485,6 +434,13 @@ struct cpe_reference *cpe_reference_new()
 	return item;
 }
 
+struct cpe_notes *cpe_notes_new(void)
+{
+	struct cpe_notes *notes = oscap_calloc(1, sizeof(struct cpe_notes));
+	notes->notes = oscap_list_new();
+	return notes;
+}
+
 struct cpe_generator *cpe_generator_new()
 {
 
@@ -613,27 +569,25 @@ struct cpe_dict_model *cpe_dict_model_parse_xml(const char *file)
 
 	__attribute__nonnull__(file);
 
-	xmlTextReaderPtr reader;
 	struct cpe_dict_model *dict = NULL;
 
 	if (!cpe_validate_xml(file))
 		return NULL;
 
-	reader = xmlReaderForFile(file, NULL, 0);
-	if (reader != NULL) {
-		xmlTextReaderNextNode(reader);
-		dict = cpe_dict_model_parse(reader);
-	} else {
-		oscap_seterr(OSCAP_EFAMILY_GLIBC, "Unable to open file: '%s'", file);
+	struct cpe_parser_ctx *ctx = cpe_parser_ctx_new(file);
+	if (ctx) {
+		xmlTextReaderNextNode(cpe_parser_ctx_get_reader(ctx));
+		dict = cpe_dict_model_parse(ctx);
 	}
-	xmlFreeTextReader(reader);
+	cpe_parser_ctx_free(ctx);
 	return dict;
 }
 
-struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
+struct cpe_dict_model *cpe_dict_model_parse(struct cpe_parser_ctx *ctx)
 {
 
-	__attribute__nonnull__(reader);
+	__attribute__nonnull__(ctx);
+	xmlTextReaderPtr reader = cpe_parser_ctx_get_reader(ctx);
 
 	struct cpe_dict_model *ret = NULL;
 	struct cpe_item *item = NULL;
@@ -662,11 +616,11 @@ struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
 	// let us figure out the base version based on the namespace of cpe-list
 	const xmlChar* nsuri = xmlTextReaderConstNamespaceUri(reader);
 
-	if (nsuri && !xmlStrcmp(nsuri, CPE_1_DICT_NS))
+	if (nsuri && !xmlStrcmp(nsuri, BAD_CAST XMLNS_CPE1D))
 	{
 		cpe_dict_model_set_base_version(ret, 1);
 	}
-	else if (nsuri && !xmlStrcmp(nsuri, CPE_2_DICT_NS)) {
+	else if (nsuri && !xmlStrcmp(nsuri, BAD_CAST XMLNS_CPE2D)) {
 		cpe_dict_model_set_base_version(ret, 2);
 	}
 	else
@@ -686,14 +640,12 @@ struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
 		}
 
 		if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_GENERATOR_STR)) {	// <generator> | count = 1
-			ret->generator = cpe_generator_parse(reader);
+			ret->generator = cpe_generator_parse(ctx);
 		} else if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CPE_ITEM_STR)) {	// <cpe-item> | cout = 0-n
-			if ((item = cpe_item_parse(reader)) == NULL) {
-				// something bad happend, let's try to recover and continue
-				// add here some bad nodes list to write it to stdout after parsing is done
-				// get the next node
-				next_ret = xmlTextReaderNextElementWE(reader, TAG_CPE_LIST_STR);
-				continue;
+			if ((item = cpe_item_parse(ctx)) == NULL) {
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Failed to parse cpe-item");
+				cpe_dict_model_free(ret);
+				return NULL;
 			}
 			// We got an item !
 			if (!cpe_dict_model_add_item(ret, item)) {
@@ -701,7 +653,6 @@ struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
 				cpe_dict_model_free(ret);
 				return NULL;
 			}
-			continue;
 		} else if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_VENDOR_STR)) {	// <vendor> | count = 0-n
 			vendor = cpe_vendor_parse(reader);
 			if (vendor)
@@ -720,10 +671,11 @@ struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
 	return ret;
 }
 
-struct cpe_generator *cpe_generator_parse(xmlTextReaderPtr reader)
+struct cpe_generator *cpe_generator_parse(struct cpe_parser_ctx *ctx)
 {
 
-	__attribute__nonnull__(reader);
+	__attribute__nonnull__(ctx);
+	xmlTextReaderPtr reader = cpe_parser_ctx_get_reader(ctx);
 
 	struct cpe_generator *ret = NULL;
 
@@ -754,13 +706,16 @@ struct cpe_generator *cpe_generator_parse(xmlTextReaderPtr reader)
 					   TAG_SCHEMA_VERSION_STR) == 0) &&
 				(xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)) {
 				ret->schema_version = (char *)xmlTextReaderReadString(reader);
+				cpe_parser_ctx_set_schema_version(ctx, ret->schema_version);
 			} else
 			    if ((xmlStrcmp(xmlTextReaderConstLocalName(reader),
 					   TAG_TIMESTAMP_STR) == 0) &&
 				(xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)) {
 				ret->timestamp = (char *)xmlTextReaderReadString(reader);
 			} else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
-				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown XML element in CPE dictionary generator");
+				oscap_seterr(OSCAP_EFAMILY_OSCAP,
+						"Unknown XML element in CPE dictionary generator, local name is '%s'.",
+						xmlTextReaderConstLocalName(reader));
 			}
 			// element saved. Let's jump on the very next one node (not element, because we need to 
 			// find XML_READER_TYPE_END_ELEMENT node, see "while" condition and the condition below "while"
@@ -773,8 +728,10 @@ struct cpe_generator *cpe_generator_parse(xmlTextReaderPtr reader)
 
 }
 
-struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
+struct cpe_item *cpe_item_parse(struct cpe_parser_ctx *ctx)
 {
+	__attribute__nonnull__(ctx);
+	xmlTextReaderPtr reader = cpe_parser_ctx_get_reader(ctx);
 
 	struct cpe_item *ret = NULL;
 	struct cpe_check *check = NULL;
@@ -782,10 +739,14 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 	char *data;
 
 	__attribute__nonnull__(reader);
-
-	if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CPE_ITEM_STR) &&
-	    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
-
+	if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CPE_ITEM_STR) != 0 ||
+			xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Found '%s'(%d) node when expecting: '%s'(%d)!",
+				xmlTextReaderConstLocalName(reader), xmlTextReaderNodeType(reader),
+				TAG_CPE_ITEM_STR, XML_READER_TYPE_ELEMENT);
+		return NULL;
+	}
+	else {
 		// we are on "<cpe-item>" element, let's alloc structure
 		ret = cpe_item_new();
 		if (ret == NULL)
@@ -801,23 +762,23 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 		// ************************************************************************************
 		data = (char *)xmlTextReaderGetAttribute(reader, ATTR_DEPRECATED_STR);
 		if (data != NULL) {	// we have a deprecation here !
+			ret->deprecated = oscap_string_to_enum(OSCAP_BOOL_MAP, data);
+			ret->export.deprecated = true;
 			oscap_free(data);
-			data = (char *)xmlTextReaderGetAttribute(reader, ATTR_DEPRECATED_BY_STR);
-			if (data == NULL || (ret->deprecated = cpe_name_new(data)) == NULL) {
-				oscap_free(ret);
-				oscap_free(data);
-				return NULL;
-			}
-			oscap_free(data);
-
-			data = (char *)xmlTextReaderGetAttribute(reader, ATTR_DEPRECATION_DATE_STR);
-			if (data == NULL || (ret->deprecation_date = oscap_alloc(strlen(data) + 1)) == NULL) {
-				oscap_free(ret);
-				oscap_free(data);
-				return NULL;
-			}
-			strcpy(ret->deprecation_date, (char *)data);
 		}
+		data = (char *)xmlTextReaderGetAttribute(reader, ATTR_DEPRECATED_BY_STR);
+		if (data != NULL) {
+			if ((ret->deprecated_by = cpe_name_new(data)) == NULL) {
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Failed to initialize CPE name with '%s'", data);
+				oscap_free(data);
+				oscap_free(ret);
+				return NULL;
+			}
+		}
+		oscap_free(data);
+
+		data = (char *)xmlTextReaderGetAttribute(reader, ATTR_DEPRECATION_DATE_STR);
+		ret->deprecation_date = oscap_strdup(data);
 		oscap_free(data);
 		// ************************************************************************************
 
@@ -833,10 +794,10 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 
 			if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_TITLE_STR) == 0) {
 				oscap_list_add(ret->titles, oscap_text_new_parse(OSCAP_TEXT_TRAITS_PLAIN, reader));
-			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTE_STR) == 0) {
-				// it's OK to use title varible here, because note is the same structure,
-				// not the reason to make a new same one
-				oscap_list_add(ret->notes, oscap_text_new_parse(OSCAP_TEXT_TRAITS_PLAIN, reader));
+			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTES_STR) == 0) {
+				struct cpe_notes *notes = cpe_notes_parse(reader);
+				if (notes != NULL)
+					oscap_list_add(ret->notes, notes);
 			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CHECK_STR) == 0) {
 				check = cpe_check_parse(reader);
 				if (check)
@@ -848,6 +809,9 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_ITEM_METADATA_STR) == 0) {
 				data = (char *)xmlTextReaderGetAttribute(reader, ATTR_MODIFICATION_DATE_STR);
 				if ((data == NULL) || ((ret->metadata = cpe_item_metadata_new()) == NULL)) {
+					oscap_seterr(OSCAP_EFAMILY_OSCAP,
+							"Failed to parse item-metadata element within cpe-item/@name='%s'",
+							cpe_name_get_as_str(ret->name));
 					cpe_item_free(ret);
 					oscap_free(data);
 					return NULL;
@@ -869,10 +833,13 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 
 			} else
 			    if ((xmlStrcmp(xmlTextReaderConstLocalName(reader),
-					   TAG_REFERENCES_STR) == 0) ||
-				(xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTES_STR) == 0)) {
+					   TAG_REFERENCES_STR) == 0)) {
 				// we just need to jump over this element
-
+			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), BAD_CAST TAG_CPE23_ITEM_STR) == 0) {
+				if ((ret->cpe23_item = cpe23_item_parse(reader)) == NULL) {
+					cpe_item_free(ret);
+					return NULL;
+				}
 			} else {
 				return ret;	// <-- we need to return here, because we don't want to jump to next element 
 			}
@@ -924,6 +891,41 @@ static struct cpe_reference *cpe_reference_parse(xmlTextReaderPtr reader)
 	return ret;
 }
 
+static struct cpe_notes *cpe_notes_parse(xmlTextReaderPtr reader)
+{
+	__attribute__nonnull__(reader);
+
+	if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTES_STR) != 0) {
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Found '%s' node when expecting: '%s'!",
+				xmlTextReaderConstLocalName(reader), TAG_NOTES_STR);
+	}
+
+	struct cpe_notes *notes = cpe_notes_new();
+	notes->lang = (char *) xmlTextReaderXmlLang(reader);
+	if (xmlTextReaderIsEmptyElement(reader) == 0) { // element contains child nodes
+		xmlTextReaderNextNode(reader);
+		while (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTES_STR) != 0) {
+			if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
+				xmlTextReaderNextNode(reader);
+				continue;
+			}
+
+			if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTE_STR) == 0) {
+				char *note_text = (char *) xmlTextReaderReadString(reader);
+				oscap_list_add(notes->notes, note_text);
+			} else {
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unexpected element within notes element: '%s'",
+						xmlTextReaderConstLocalName(reader));
+				cpe_notes_free(notes);
+				return NULL;
+			}
+			xmlTextReaderNextNode(reader);
+		}
+	}
+
+	return notes;
+}
+
 struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 {
 
@@ -973,6 +975,7 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 				else if (strcasecmp((const char *)data, "a") == 0)
 				    product->part = CPE_PART_APP;
 				else {
+					oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown attribute value of vendor/@part='%s'", data);
 				    oscap_free(ret);
 				    oscap_free(data);
 				    return NULL;
@@ -1003,6 +1006,9 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 			language = cpe_language_new();
 			language->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 			oscap_list_add(edition->languages, language);
+		} else {
+			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown XML element withinin CPE vendor element, local name is '%s'.",
+				xmlTextReaderConstLocalName(reader));
 		}
 		xmlTextReaderNextNode(reader);
 	}
@@ -1056,10 +1062,10 @@ void cpe_dict_export(const struct cpe_dict_model *dict, xmlTextWriterPtr writer)
 
 	switch (base_version) {
 		case 1:
-			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_LIST_STR, CPE_1_DICT_NS);
+			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_LIST_STR, BAD_CAST XMLNS_CPE1D);
 			break;
 		case 2:
-			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_LIST_STR, CPE_2_DICT_NS);
+			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_LIST_STR, BAD_CAST XMLNS_CPE2D);
 			break;
 		default:
 			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown CPE base version '%i'.", base_version);
@@ -1067,11 +1073,11 @@ void cpe_dict_export(const struct cpe_dict_model *dict, xmlTextWriterPtr writer)
 			break;
 	}
 
-	xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:meta", CPEMETA_NS);
+	xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:meta", BAD_CAST XMLNS_CPE2D_METADATA);
 	xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:xsi", BAD_CAST "http://www.w3.org/2001/XMLSchema-instance");
 	xmlTextWriterWriteAttribute(writer, BAD_CAST "xsi:schemaLocation", BAD_CAST
-			"http://scap.nist.gov/schema/cpe-dictionary-metadata/0.2 http://nvd.nist.gov/schema/cpe-dictionary-metadata_0.2.xsd "
-			"http://cpe.mitre.org/dictionary/2.0 http://cpe.mitre.org/files/cpe-dictionary_2.1.xsd");
+			XMLNS_CPE2D_METADATA " http://nvd.nist.gov/schema/cpe-dictionary-metadata_0.2.xsd "
+			XMLNS_CPE2D " http://cpe.mitre.org/files/cpe-dictionary_2.1.xsd");
 
 	if (dict->generator) cpe_generator_export(dict->generator, writer);
 	OSCAP_FOREACH(cpe_item, item, cpe_dict_model_get_items(dict),
@@ -1080,9 +1086,11 @@ void cpe_dict_export(const struct cpe_dict_model *dict, xmlTextWriterPtr writer)
 
 	if (base_version >= 2) {
 	    // TODO: NEED TO HAVE COMPONENT-TREE STRUCTURE TO GET XML-NAMESPACE 
-	    xmlTextWriterStartElementNS(writer, NULL, TAG_COMPONENT_TREE_STR, CPEMETA_NS);
-		OSCAP_FOREACH(cpe_vendor, vendor, cpe_dict_model_get_vendors(dict), cpe_vendor_export(vendor, writer);)
-	    xmlTextWriterEndElement(writer);	//</component-tree>
+		if (oscap_list_get_itemcount(dict->vendors) != 0) {
+			xmlTextWriterStartElementNS(writer, NULL, TAG_COMPONENT_TREE_STR, BAD_CAST XMLNS_CPE2D_METADATA);
+			OSCAP_FOREACH(cpe_vendor, vendor, cpe_dict_model_get_vendors(dict), cpe_vendor_export(vendor, writer);)
+			xmlTextWriterEndElement(writer);	//</component-tree>
+		}
 	}
 
 	xmlTextWriterEndElement(writer);
@@ -1134,10 +1142,10 @@ void cpe_item_export(const struct cpe_item *item, xmlTextWriterPtr writer, int b
 
 	switch (base_version) {
 		case 1:
-			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_ITEM_STR, CPE_1_DICT_NS);
+			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_ITEM_STR, BAD_CAST XMLNS_CPE1D);
 			break;
 		case 2:
-			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_ITEM_STR, CPE_2_DICT_NS);
+			xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_ITEM_STR, BAD_CAST XMLNS_CPE2D);
 			break;
 		default:
 			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown CPE base version '%i'.", base_version);
@@ -1150,16 +1158,28 @@ void cpe_item_export(const struct cpe_item *item, xmlTextWriterPtr writer, int b
 		xmlTextWriterWriteAttribute(writer, ATTR_NAME_STR, BAD_CAST temp);
 		oscap_free(temp);
 	}
-	if (item->deprecated != NULL) {
-		temp = cpe_name_get_as_format(item->deprecated, CPE_FORMAT_URI);
-		xmlTextWriterWriteAttribute(writer, ATTR_DEPRECATED_STR, VAL_TRUE_STR);
-		xmlTextWriterWriteAttribute(writer, ATTR_DEPRECATION_DATE_STR, BAD_CAST item->deprecation_date);
+	if (item->export.deprecated == true) {
+		xmlTextWriterWriteAttribute(writer, ATTR_DEPRECATED_STR,
+			BAD_CAST oscap_enum_to_string(OSCAP_BOOL_MAP, item->deprecated));
+	}
+	if (item->deprecated_by != NULL) {
+		temp = cpe_name_get_as_format(item->deprecated_by, CPE_FORMAT_URI);
 		xmlTextWriterWriteAttribute(writer, ATTR_DEPRECATED_BY_STR, BAD_CAST temp);
 		oscap_free(temp);
 	}
+	if (item->deprecation_date != NULL) {
+		xmlTextWriterWriteAttribute(writer, ATTR_DEPRECATION_DATE_STR, BAD_CAST item->deprecation_date);
+	}
 
 	oscap_textlist_export(cpe_item_get_titles(item), writer, "title");
 
+	it = oscap_iterator_new(item->notes);
+	if (oscap_iterator_has_more(it)) {
+		cpe_notes_export(oscap_iterator_next(it), writer);
+	}
+
+	oscap_iterator_free(it);
+
 	    if (item->metadata != NULL) {
 		xmlTextWriterStartElementNS(writer, NS_META_STR, TAG_ITEM_METADATA_STR, NULL);
 		if (item->metadata->modification_date != NULL)
@@ -1183,10 +1203,10 @@ void cpe_item_export(const struct cpe_item *item, xmlTextWriterPtr writer, int b
 	}
 	oscap_iterator_free(it);
 
-	oscap_textlist_export(cpe_item_get_notes(item), writer, "title");
-
 	OSCAP_FOREACH(cpe_check, check, cpe_item_get_checks(item), cpe_check_export(check, writer);)
-
+	if (item->cpe23_item != NULL) {
+		cpe23_item_export(item->cpe23_item, writer);
+	}
 	    xmlTextWriterEndElement(writer);
 	if (xmlGetLastError() != NULL)
 		oscap_setxmlerr(xmlGetLastError());
@@ -1321,6 +1341,26 @@ static void cpe_reference_export(const struct cpe_reference *ref, xmlTextWriterP
 	xmlTextWriterEndElement(writer);
 }
 
+static void cpe_notes_export(const struct cpe_notes *notes, xmlTextWriterPtr writer)
+{
+	__attribute__nonnull__(notes);
+	__attribute__nonnull__(writer);
+
+	xmlTextWriterStartElementNS(writer, NULL, TAG_NOTES_STR, NULL);
+	if (notes->lang != NULL)
+		xmlTextWriterWriteAttribute(writer, BAD_CAST "xml:lang", BAD_CAST notes->lang);
+	struct oscap_iterator *it = oscap_iterator_new(notes->notes);
+	while (oscap_iterator_has_more(it)) {
+		const char *note_text = oscap_iterator_next(it);
+		xmlTextWriterStartElementNS(writer, NULL, TAG_NOTE_STR, NULL);
+		if (note_text != NULL)
+			xmlTextWriterWriteString(writer, BAD_CAST note_text);
+		xmlTextWriterEndElement(writer);
+	}
+	oscap_iterator_free(it);
+	xmlTextWriterEndElement(writer);
+}
+
 /* End of private export functions
  * */
 /***************************************************************************/
@@ -1346,13 +1386,14 @@ void cpe_item_free(struct cpe_item *item)
 	if (item == NULL)
 		return;
 	cpe_name_free(item->name);
-	cpe_name_free(item->deprecated);
+	cpe_name_free(item->deprecated_by);
 	oscap_free(item->deprecation_date);
 	oscap_list_free(item->references, (oscap_destruct_func) cpe_reference_free);
 	oscap_list_free(item->checks, (oscap_destruct_func) cpe_check_free);
-	oscap_list_free(item->notes, (oscap_destruct_func) oscap_text_free);
+	oscap_list_free(item->notes, (oscap_destruct_func) cpe_notes_free);
 	oscap_list_free(item->titles, (oscap_destruct_func) oscap_text_free);
 	cpe_itemmetadata_free(item->metadata);
+	cpe23_item_free(item->cpe23_item);
 	oscap_free(item);
 }
 
@@ -1392,6 +1433,15 @@ void cpe_reference_free(struct cpe_reference *ref)
 	oscap_free(ref);
 }
 
+void cpe_notes_free(struct cpe_notes *notes)
+{
+	if (notes != NULL) {
+		oscap_list_free(notes->notes, (oscap_destruct_func) oscap_free);
+		oscap_free(notes->lang);
+		oscap_free(notes);
+	}
+}
+
 void cpe_vendor_free(struct cpe_vendor *vendor)
 {
 
