@@ -44,6 +44,8 @@
 #include "common/text_priv.h"
 #include "common/elements.h"
 #include "common/_error.h"
+#include "common/xmlns_priv.h"
+#include "common/xmltext_priv.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -102,7 +104,6 @@ OSCAP_GETTER(const struct cpe_testexpr*, cpe_platform, expr)
 #define VAL_OR_STR      BAD_CAST "OR"
 #define VAL_FALSE_STR   BAD_CAST "false"
 #define VAL_TRUE_STR    BAD_CAST "true"
-#define CPELANG_NS      BAD_CAST "http://cpe.mitre.org/language/2.0"
 /* End of XML string variables definitions
  * */
 /***************************************************************************/
@@ -111,87 +112,13 @@ OSCAP_GETTER(const struct cpe_testexpr*, cpe_platform, expr)
  * These function shoud not be called from outside. For exporting these elements
  * has to call parent element's 
  */
-static int xmlTextReaderNextElement(xmlTextReaderPtr reader);
 static char *parse_text_element(xmlTextReaderPtr reader, char *name);
 static bool cpe_validate_xml(const char *filename);
-static int xmlTextReaderNextNode(xmlTextReaderPtr reader);
 
 /* End of static declarations 
  * */
 /***************************************************************************/
 
-/* Function testing reader function 
- */
-static int xmlTextReaderNextNode(xmlTextReaderPtr reader)
-{
-
-	__attribute__nonnull__(reader);
-
-	int ret;
-	ret = xmlTextReaderRead(reader);
-	if (ret == -1) {
-		oscap_setxmlerr(xmlCtxtGetLastError(reader));
-		/* TODO: Should we end here as fatal ? */
-	}
-
-	return ret;
-}
-
-/* Function that jump to next XML starting element.
- */
-static int xmlTextReaderNextElement(xmlTextReaderPtr reader)
-{
-
-	__attribute__nonnull__(reader);
-
-	int ret;
-	do {
-		ret = xmlTextReaderRead(reader);
-		// if end of file
-		if (ret < 1)
-			break;
-	} while (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
-
-	if (ret == -1) {
-		oscap_setxmlerr(xmlCtxtGetLastError(reader));
-		/* TODO: Should we end here as fatal ? */
-	}
-
-	return ret;
-}
-
-/* Function that jump to next XML starting element.
- *
- * This function makes sure we don't go past end tag of given element
- * */
-static int xmlTextReaderNextElementWE(xmlTextReaderPtr reader, xmlChar* end_tag)
-{
-
-	__attribute__nonnull__(reader);
-
-	int ret;
-	do {
-		ret = xmlTextReaderRead(reader);
-		// if end of file
-		if (ret < 1)
-			break;
-
-		if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT) {
-			if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), end_tag)) {
-				ret = 0;
-				break;
-			}
-		}
-	} while (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
-
-	if (ret == -1) {
-		oscap_setxmlerr(xmlCtxtGetLastError(reader));
-		/* TODO: Should we end here as fatal ? */
-	}
-
-	return ret;
-}
-
 const struct cpe_testexpr *cpe_testexpr_get_next(const struct cpe_testexpr *expr)
 {
 
@@ -606,7 +533,7 @@ void cpe_lang_export(const struct cpe_lang_model *spec, xmlTextWriterPtr writer)
 	__attribute__nonnull__(spec);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, NULL, TAG_PLATFORM_SPEC_STR, CPELANG_NS);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_PLATFORM_SPEC_STR, BAD_CAST XMLNS_CPE2L);
 		OSCAP_FOREACH(cpe_platform, p, cpe_lang_model_get_platforms(spec),
 			      // dump its contents to XML tree
 			      cpe_platform_export(p, writer);)
@@ -651,7 +578,7 @@ void cpe_testexpr_export(const struct cpe_testexpr *expr, xmlTextWriterPtr write
 		xmlTextWriterEndElement(writer);
 	}
 	else {
-		xmlTextWriterStartElementNS(writer, NULL, TAG_LOGICAL_TEST_STR, CPELANG_NS);
+		xmlTextWriterStartElementNS(writer, NULL, TAG_LOGICAL_TEST_STR, BAD_CAST XMLNS_CPE2L);
 
 		if (expr->oper == CPE_LANG_OPER_AND) {
 			xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_AND_STR);
