@@ -111,7 +111,7 @@ static xmlNodePtr _lookup_request_in_arf(xmlDocPtr doc, const char *request_id)
 			continue;
 
 		char* candidate_id = (char*)xmlGetProp(candidate, BAD_CAST "id");
-		if (strcmp(candidate_id, request_id) == 0)
+		if (request_id == NULL || strcmp(candidate_id, request_id) == 0)
 		{
 			component = candidate;
 			xmlFree(candidate_id);
@@ -123,7 +123,7 @@ static xmlNodePtr _lookup_request_in_arf(xmlDocPtr doc, const char *request_id)
 	return component;
 }
 
-static int ds_rds_dump_arf_content(xmlDocPtr doc, xmlNodePtr parent_node, const char* target_file)
+static xmlNodePtr ds_rds_get_inner_content(xmlDocPtr doc, xmlNodePtr parent_node)
 {
 	xmlNodePtr candidate = parent_node->children;
 	xmlNodePtr content_node = NULL;
@@ -141,10 +141,20 @@ static int ds_rds_dump_arf_content(xmlDocPtr doc, xmlNodePtr parent_node, const
 
 	if (!content_node) {
 		oscap_seterr(OSCAP_EFAMILY_XML, "Given ARF node has no 'arf:content' node inside!");
-		return -1;
+		return NULL;
 	}
 
-	candidate = content_node->children;
+	return content_node;
+}
+
+static int ds_rds_dump_arf_content(xmlDocPtr doc, xmlNodePtr parent_node, const char* target_file)
+{
+	xmlNodePtr content_node = ds_rds_get_inner_content(doc, parent_node);
+
+	if (!content_node)
+		return -1;
+
+	xmlNodePtr candidate = content_node->children;
 	xmlNodePtr inner_root = NULL;
 
 	for (; candidate != NULL; candidate = candidate->next)
@@ -290,82 +300,6 @@ static void ds_rds_add_relationship(xmlDocPtr doc, xmlNodePtr relationships,
 	xmlAddChild(relationships, relationship);
 }
 
-// Asset Identification specification requires fully expanded IPv6 addresses.
-// Despite this being greatly discouraged by RFC 5952 we have to comply and
-// expand given IPv6, otherwise the resulting AI content would be invalid.
-static char* ds_rds_expand_ip_v6(const char* input)
-{
-	// we have to do a double pass because we need to know the number of
-	// specified components, :: will fill up to a total of 8 if encountered
-
-	const char* input_it = input;
-	unsigned short component_count = 1; // fencepost, we start with 1
-	while (*input_it)
-	{
-		if (*input_it == ':')
-		{
-			// don't count ::1 as 2 components
-			if (input_it != input)
-				component_count++;
-
-			// the next character is either a part of a component or another ':'
-			// in any case we don't want to count it
-			input_it++;
-		}
-
-		input_it++;
-	}
-
-	// IPv6 is at most eight 4-tuples of [0-9a-f] with 7 separators, plus \0
-	char* ret = oscap_alloc(8 * 4 * sizeof(char) + 7 + 1);
-	char* output_it = ret;
-
-	input_it = input;
-	while (*input_it)
-	{
-		// signifies that we have finished writing a component this iteration
-		bool closed_component = false;
-
-		if (*input_it == ':')
-		{
-			const char next = *(input_it + 1 * sizeof(char));
-			if (next == ':')
-			{
-				// skip one extra char to skip over the whole ::
-				input_it++;
-
-				for (unsigned short i = component_count; i < 8; ++i)
-				{
-					// don't add leading separators
-					if (output_it != ret)
-						*output_it++ = ':';
-
-					*output_it++ = '0';
-					closed_component = true;
-				}
-			}
-			else
-			{
-				closed_component = true;
-			}
-		}
-		else
-		{
-			*output_it++ = *input_it;
-		}
-
-		input_it++;
-
-		// if this was not the last char of the input add a separator
-		if (closed_component && *input_it)
-			*output_it++ = ':';
-	}
-
-	*output_it = '\0';
-
-	return ret;
-}
-
 static xmlNodePtr ds_rds_add_ai_from_xccdf_results(xmlDocPtr doc, xmlNodePtr assets,
 		xmlDocPtr xccdf_result_doc)
 {
@@ -448,7 +382,7 @@ static xmlNodePtr ds_rds_add_ai_from_xccdf_results(xmlDocPtr doc, xmlNodePtr ass
 			else // IPv6 has semicolons instead of dots
 			{
 				// lets expand the IPv6 to conform to the AI XSD and specification
-				char* expanded_ipv6 = ds_rds_expand_ip_v6((const char*)content);
+				char *expanded_ipv6 = oscap_expand_ipv6((const char*)content);
 				xmlNewTextChild(ip_address, ai_ns, BAD_CAST "ip-v6", BAD_CAST expanded_ipv6);
 				oscap_free(expanded_ipv6);
 			}
@@ -459,6 +393,148 @@ static xmlNodePtr ds_rds_add_ai_from_xccdf_results(xmlDocPtr doc, xmlNodePtr ass
 	return asset;
 }
 
+static int ds_rds_report_inject_ai_target_id_ref(xmlDocPtr doc, xmlNodePtr report, const char *asset_id)
+{
+	xmlNodePtr content_node = ds_rds_get_inner_content(doc, report);
+
+	if (!content_node)
+		return -1;
+
+	if (!content_node->children) {
+		oscap_seterr(OSCAP_EFAMILY_XML, "Given report doesn't contain any data, "
+			"can't inject AI asset target id ref");
+		return -1;
+	}
+
+	xmlNodePtr test_result_node = NULL;
+	xmlNodePtr test_result_candidate = content_node->children;
+	xmlNodePtr inner_element_node = NULL;
+
+	while (test_result_candidate) {
+		if (test_result_candidate->type == XML_ELEMENT_NODE) {
+			inner_element_node = test_result_candidate;
+
+			if (strcmp((const char*)test_result_candidate->name, "TestResult") == 0) {
+				test_result_node = test_result_candidate;
+				break;
+			}
+		}
+
+		test_result_candidate = test_result_candidate->next;
+	}
+
+	if (!inner_element_node) {
+		oscap_seterr(OSCAP_EFAMILY_XML, "Given report doesn't contain any XML element! "
+			"Can't inject AI asset target id ref");
+		return -1;
+	}
+
+	if (!test_result_node) {
+		// TestResult may not be the top level element in the report.
+		// While that is very unusual it is legitimate, lets check child elements.
+
+		// As a rule, we only inject target-id-ref to the last test result
+		// (XML, top-down).
+
+		if (strcmp((const char*)inner_element_node->name, "Benchmark")) {
+			oscap_seterr(OSCAP_EFAMILY_XML, "Top level element of the report isn't TestResult "
+				"or Benchmark, the report is likely invalid!");
+			return -1;
+		}
+
+		if (!inner_element_node->children) {
+			oscap_seterr(OSCAP_EFAMILY_XML, "Top level element of the report isn't TestResult "
+				"and does not contain any children! No TestResult to inject to has been found.");
+			return -1;
+		}
+
+		test_result_candidate = inner_element_node->children;
+		while (test_result_candidate) {
+			if (test_result_candidate->type == XML_ELEMENT_NODE) {
+				if (strcmp((const char*)test_result_candidate->name, "TestResult") == 0) {
+					test_result_node = test_result_candidate;
+					// we intentionally do not break here, we are looking for the
+					// last (top-down) TestResult in the report.
+					//break;
+				}
+			}
+
+			test_result_candidate = test_result_candidate->next;
+		}
+	}
+
+	if (!test_result_node) {
+		oscap_seterr(OSCAP_EFAMILY_XML, "TestResult node to inject to has not been found"
+			"(checked root element and all children of it).");
+		return -1;
+	}
+
+	// Now we need to find the right place to inject the target-id-ref element.
+	// It has to come after target, target-address and target-facts elements.
+	// However target-address and target-fact are both optional.
+
+	xmlNodePtr prev_sibling = NULL;
+	xmlNodePtr prev_sibling_candidate = test_result_node->children;
+
+	while (prev_sibling_candidate) {
+		if (prev_sibling_candidate->type == XML_ELEMENT_NODE) {
+			if (strcmp((const char*)prev_sibling_candidate->name, "target") == 0 ||
+				strcmp((const char*)prev_sibling_candidate->name, "target-address") == 0 ||
+				strcmp((const char*)prev_sibling_candidate->name, "target-facts") == 0) {
+
+				prev_sibling = prev_sibling_candidate;
+			}
+		}
+
+		prev_sibling_candidate = prev_sibling_candidate->next;
+	}
+
+	if (!prev_sibling) {
+		oscap_seterr(OSCAP_EFAMILY_XML, "No target element was found in TestResult. "
+			"The most likely reason is that the content is not valid! "
+			"(XCCDF spec states 'target' element as required)");
+		return -1;
+	}
+
+	// We have to make sure we are not injecting a target-id-ref that is there
+	// already. if there is any duplicate, it has to come right after prev_sibling.
+	xmlNodePtr duplicate_candidate = prev_sibling->next;
+	while (duplicate_candidate) {
+		if (duplicate_candidate->type == XML_ELEMENT_NODE) {
+			if (strcmp((const char*)duplicate_candidate->name, "target-id-ref") == 0) {
+				xmlChar* system_attr = xmlGetProp(duplicate_candidate, BAD_CAST "system");
+				xmlChar* name_attr = xmlGetProp(duplicate_candidate, BAD_CAST "name");
+
+				if (strcmp((const char*)system_attr, ai_ns_uri) == 0 &&
+					strcmp((const char*)name_attr, asset_id) == 0) {
+
+					xmlFree(system_attr);
+					xmlFree(name_attr);
+					return 0;
+				}
+
+				xmlFree(system_attr);
+				xmlFree(name_attr);
+			}
+			else {
+				break;
+			}
+		}
+		duplicate_candidate = duplicate_candidate->next;
+	}
+
+	xmlNodePtr target_id_ref = xmlNewNode(prev_sibling->ns, BAD_CAST "target-id-ref");
+	xmlNewProp(target_id_ref, BAD_CAST "system", BAD_CAST ai_ns_uri);
+	xmlNewProp(target_id_ref, BAD_CAST "name", BAD_CAST asset_id);
+	// @href is a required attribute by the XSD! The spec advocates filling it
+	// blank when it's not needed.
+	xmlNewProp(target_id_ref, BAD_CAST "href", BAD_CAST "");
+
+	xmlAddNextSibling(prev_sibling, target_id_ref);
+
+	return 0;
+}
+
 static void ds_rds_add_xccdf_test_results(xmlDocPtr doc, xmlNodePtr reports,
 		xmlDocPtr xccdf_result_file_doc, xmlNodePtr relationships, xmlNodePtr assets,
 		const char* report_request_id)
@@ -472,7 +548,7 @@ static void ds_rds_add_xccdf_test_results(xmlDocPtr doc, xmlNodePtr reports,
 	// be done with it.
 	if (strcmp((const char*)root_element->name, "TestResult") == 0)
 	{
-		ds_rds_create_report(doc, reports, xccdf_result_file_doc, "xccdf1");
+		xmlNodePtr report = ds_rds_create_report(doc, reports, xccdf_result_file_doc, "xccdf1");
 		ds_rds_add_relationship(doc, relationships, "arfvocab:createdFor",
 				"xccdf1", report_request_id);
 
@@ -481,6 +557,10 @@ static void ds_rds_add_xccdf_test_results(xmlDocPtr doc, xmlNodePtr reports,
 		ds_rds_add_relationship(doc, relationships, "arfvocab:isAbout",
 				"xccdf1", asset_id);
 		xmlFree(asset_id);
+
+		// We deliberately don't act on errors in inject ai target-id-ref as
+		// these aren't fatal errors.
+		ds_rds_report_inject_ai_target_id_ref(doc, report, asset_id);
 	}
 
 	// 2) the root element is a Benchmark, TestResults are embedded within
@@ -511,7 +591,7 @@ static void ds_rds_add_xccdf_test_results(xmlDocPtr doc, xmlNodePtr reports,
 			xmlDOMWrapFreeCtxt(wrap_ctxt);
 
 			char* report_id = oscap_sprintf("xccdf%i", report_suffix++);
-			ds_rds_create_report(doc, reports, wrap_doc, report_id);
+			xmlNodePtr report = ds_rds_create_report(doc, reports, wrap_doc, report_id);
 			ds_rds_add_relationship(doc, relationships, "arfvocab:createdFor",
 					report_id, report_request_id);
 
@@ -519,6 +599,11 @@ static void ds_rds_add_xccdf_test_results(xmlDocPtr doc, xmlNodePtr reports,
 			char* asset_id = (char*)xmlGetProp(asset, BAD_CAST "id");
 			ds_rds_add_relationship(doc, relationships, "arfvocab:isAbout",
 					report_id, asset_id);
+
+			// We deliberately don't act on errors in inject ai target-id-ref as
+			// these aren't fatal errors.
+			ds_rds_report_inject_ai_target_id_ref(doc, report, asset_id);
+
 			xmlFree(asset_id);
 
 			oscap_free(report_id);
