@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009-2011 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2013 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -40,8 +40,8 @@
 #include <time.h>
 
 #include "oval_definitions_impl.h"
-#include "oval_collection_impl.h"
-#include "oval_string_map_impl.h"
+#include "adt/oval_collection_impl.h"
+#include "adt/oval_string_map_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_parser_impl.h"
 #include "oval_probe_session.h"
