@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2013 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,9 @@
 
 #include <string.h>
 
+#include "oval_agent_api_impl.h"
 #include "oval_definitions_impl.h"
-#include "oval_string_map_impl.h"
+#include "adt/oval_string_map_impl.h"
 #include "oval_system_characteristics_impl.h"
 #include "oval_results_impl.h"
 #include "common/util.h"
