@@ -34,7 +34,7 @@
 #include <stdlib.h>
 
 #include "common/_error.h"
-#include "common/public/alloc.h"
+#include "common/alloc.h"
 #include "common/assume.h"
 #include "common/util.h"
 #include "common/bfind.h"
@@ -250,8 +250,8 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 	SEXP_free(ret_code);
 
 	if (oscap_err()) {
-		oscap_dlprintf(DBG_E, "Failed: id: %s, err: %d, %d, %s.\n",
-			       id_str, oscap_err_family(), oscap_err_code(), oscap_err_desc());
+		oscap_dlprintf(DBG_E, "Failed: id: %s, err: %d, %s.\n",
+			       id_str, oscap_err_family(), oscap_err_desc());
 		oscap_clearerr();
 		oscap_free(id_str);
 		SEXP_free(ret);
@@ -350,9 +350,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                         }
 
                                         if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECONN, "Can't connect to the probe");
+                                                oscap_seterr (OSCAP_EFAMILY_OVAL, "Can't connect to the probe");
                                         else
-                                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECONN, errbuf);
+                                                oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 					return (-1);
 				}
@@ -369,7 +369,7 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 }
 
                                 SEAP_msg_free(s_omsg);
-                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBEUNKNOWN, NULL);
+                                oscap_seterr (OSCAP_EFAMILY_OVAL, "OVAL_EPROBEUNKNOWN");
 
 				return (-1);
 			}
@@ -387,9 +387,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 char errbuf[__ERRBUF_SIZE];
 
                                 if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, "Unable to send a message to probe");
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, "Unable to send a message to probe");
                                 else
-					oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, errbuf);
+					oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 				SEAP_msg_free(s_omsg);
 				return (-1);
@@ -404,9 +404,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 }
 
                                 if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, "Can't close sd");
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, "Can't close sd");
                                 else
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, errbuf);
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 				pd->sd = -1;
 				return (-1);
@@ -426,9 +426,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 }
 
                                 if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, "Unable to send a message to probe");
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, "Unable to send a message to probe");
                                 else
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, errbuf);
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 				return (ret);
 			}
@@ -460,14 +460,12 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 						break;
 					case  1: /* no error found */
 						dE("Internal error: An error was signaled on sd=%d but the error queue is empty.\n");
-						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE,
-							     "SEAP_recverr_byid: internal error: empty error queue.");
+						oscap_seterr(OSCAP_EFAMILY_OVAL, "SEAP_recverr_byid: internal error: empty error queue.");
 						SEAP_msg_free(s_omsg);
 						return (-1);
 					case -1: /* internal error */
 						dE("Internal error: SEAP_recverr_byid returned -1\n");
-						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE,
-							     "SEAP_recverr_byid: internal error.");
+						oscap_seterr(OSCAP_EFAMILY_OVAL, "SEAP_recverr_byid: internal error.");
 						SEAP_msg_free(s_omsg);
 						return (-1);
 					}
@@ -478,7 +476,7 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 					switch (err->type) {
 					case SEAP_ETYPE_USER:
 					{
-						char errmsg[512] = "", *codemsg;
+						char * codemsg;
 
 						/*
 						 * Errors of type USER should all be from the probe "namespace" (i.e. only codes
@@ -518,13 +516,12 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 							codemsg = "Unknown error";
 						}
 
-						snprintf(errmsg, sizeof errmsg, "probe at sd=%d reported an error: %s", pd->sd, codemsg);
-						dE("Received an error from probe at sd=%d: %u, \"%s\"\n", pd->sd, err->code, errmsg);
-						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE, errmsg);
+						oscap_seterr(OSCAP_EFAMILY_OVAL, "Probe at sd=%d (%s) reported an error: %s",
+							     pd->sd, oval_subtype_to_str(pd->subtype), codemsg);
 						break;
 					}
 					case SEAP_ETYPE_INT:
-						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE, "Internal error");
+						oscap_seterr(OSCAP_EFAMILY_OVAL, "Internal error");
 						break;
 					}
 
@@ -540,9 +537,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 				char errbuf[__ERRBUF_SIZE];
 
                                 if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, "Unable to receive a message to probe");
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, "Unable to receive a message to probe");
                                 else
-					oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, errbuf);
+					oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 				SEAP_msg_free(s_imsg);
 				SEAP_msg_free(s_omsg);
@@ -560,9 +557,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 }
 
                                 if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, "Unable to close probe sd");
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, "Unable to close probe sd");
                                 else
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, errbuf);
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 				pd->sd = -1;
 				return (-1);
@@ -584,9 +581,9 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 					}
 
 					if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-						oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBERECV, "Unable to receive a message from probe");
+						oscap_seterr (OSCAP_EFAMILY_OVAL, "Unable to receive a message from probe");
 					else
-						oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBERECV, errbuf);
+						oscap_seterr (OSCAP_EFAMILY_OVAL, errbuf);
 
 					return (ret);
 				}
@@ -679,12 +676,12 @@ static int oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct oval_sysch
                 val = probe_obj_getentval (obj, __STRING(name), 1);     \
                                                                         \
                 if (val == NULL) {                                      \
-                        _D("No entity or value: %s\n", __STRING(name)); \
+                        dI("No entity or value: %s\n", __STRING(name)); \
                         goto fail;                                      \
                 }                                                       \
                                                                         \
                 if (SEXP_string_cstr_r (val, buf, sizeof buf) >= sizeof buf) { \
-                        _D("Value too large: %s\n", __STRING(name));    \
+                        dI("Value too large: %s\n", __STRING(name));    \
                         SEXP_free (val);                                \
                         goto fail;                                      \
                 }                                                       \
@@ -715,12 +712,12 @@ static int oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct oval_sysch
                                 val = probe_ent_getattrval (ent, __STRING(name)); \
                                                                         \
                                 if (val == NULL) {                      \
-                                        _D("No value: %s\n", __STRING(name)); \
+                                        dI("No value: %s\n", __STRING(name)); \
                                         goto fail;                      \
                                 }                                       \
                                                                         \
                                 if (SEXP_string_cstr_r (val, buf, sizeof buf) >= sizeof buf) { \
-                                        _D("Value too large: %s\n", __STRING(name)); \
+                                        dI("Value too large: %s\n", __STRING(name)); \
                                         SEXP_free (val);                \
                                         goto fail;                      \
                                 }                                       \
@@ -766,11 +763,10 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
         switch(act) {
         case PROBE_HANDLER_ACT_EVAL:
         {
-                struct oval_object   *obj;
                 struct oval_sysinfo **inf;
                 /* int flags = va_arg(ap, int); */
 
-                obj = va_arg(ap, struct oval_object *);
+                va_arg(ap, struct oval_object *);
                 inf = va_arg(ap, struct oval_sysinfo **);
                 pd  = oval_pdtbl_get(pext->pdtbl, type);
 
@@ -789,7 +785,7 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
         }
         case PROBE_HANDLER_ACT_OPEN:
         {
-                char         probe_uri[PATH_MAX + 1], errmsg[__ERRBUF_SIZE];
+                char         probe_uri[PATH_MAX + 1];
                 size_t       probe_urilen;
                 char        *probe_dir;
                 oval_pdsc_t *probe_dsc;
@@ -798,8 +794,7 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
                 probe_dsc = oval_pdsc_lookup(pext->pdsc, pext->pdsc_cnt, type);
 
 		if (probe_dsc == NULL) {
-			snprintf (errmsg, sizeof errmsg, "subtype %u not supported", type);
-			oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
+			oscap_seterr (OSCAP_EFAMILY_OVAL, "subtype %u not supported", type);
 
 			ret = -1;
 			break;
@@ -809,16 +804,14 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
                                         "%s://%s/%s", OVAL_PROBE_SCHEME, probe_dir, probe_dsc->file);
 
                 if (probe_urilen >= sizeof probe_uri) {
-                        snprintf (errmsg, sizeof errmsg, "probe URI too long");
-                        oscap_seterr (OSCAP_EFAMILY_GLIBC, ENAMETOOLONG, errmsg);
+                        oscap_seterr (OSCAP_EFAMILY_GLIBC, "probe URI too long");
 
                         ret = -1;
                 } else {
                         oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
 
                         if (oval_pdtbl_add(pext->pdtbl, type, -1, probe_uri) != 0) {
-                                snprintf (errmsg, sizeof errmsg, "%s probe not supported", probe_dsc->name);
-                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
+                                oscap_seterr (OSCAP_EFAMILY_OVAL, "%s probe not supported", probe_dsc->name);
 
                                 ret = -1;
                         }
@@ -859,7 +852,7 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                 pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
 
                 if (pd == NULL) {
-                        char         probe_uri[PATH_MAX + 1], errmsg[__ERRBUF_SIZE];
+                        char         probe_uri[PATH_MAX + 1];
                         size_t       probe_urilen;
                         char        *probe_dir;
                         oval_pdsc_t *probe_dsc;
@@ -878,8 +871,7 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                                                 "%s://%s/%s", OVAL_PROBE_SCHEME, probe_dir, probe_dsc->file);
 
                         if (probe_urilen >= sizeof probe_uri) {
-                                snprintf (errmsg, sizeof errmsg, "probe URI too long");
-                                oscap_seterr (OSCAP_EFAMILY_GLIBC, ENAMETOOLONG, errmsg);
+                                oscap_seterr (OSCAP_EFAMILY_GLIBC, "probe URI too long");
 				va_end(ap);
                                 return (-1);
                         }
@@ -896,7 +888,7 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
 			pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
 
                         if (pd == NULL) {
-                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "internal error");
+                                oscap_seterr (OSCAP_EFAMILY_OVAL, "internal error");
 				va_end(ap);
                                 return (-1);
                         }
@@ -935,14 +927,12 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
         case PROBE_HANDLER_ACT_RESET:
 	case PROBE_HANDLER_ACT_ABORT:
         {
-                size_t i;
-
                 if (type == OVAL_SUBTYPE_ALL) {
                         /*
                          * Iterate thru probe descriptor table and execute the reset operation
                          * for each probe descriptor.
                          */
-                        for (i = 0; i < pext->pdtbl->count; ++i) {
+                        for (size_t i = 0; i < pext->pdtbl->count; ++i) {
                                 pd  = pext->pdtbl->memb[i];
 
 				if (pd == NULL) {
@@ -1077,7 +1067,7 @@ int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struc
 	int ret;
 
 	if (syschar == NULL) {
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBEINVAL, "Internal error: syschar == NULL");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Internal error: syschar == NULL");
 		return (-1);
 	}
 
@@ -1125,9 +1115,7 @@ int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struc
 
 int oval_probe_ext_reset(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
 {
-        void *res;
-
-        res = SEAP_cmd_exec(ctx, pd->sd, SEAP_EXEC_RECV, PROBECMD_RESET, NULL, SEAP_CMDTYPE_SYNC, NULL, NULL);
+        SEAP_cmd_exec(ctx, pd->sd, SEAP_EXEC_RECV, PROBECMD_RESET, NULL, SEAP_CMDTYPE_SYNC, NULL, NULL);
 
         return (0);
 }
