@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2013 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -26,6 +26,7 @@
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  *      "Peter Vrabec" <pvrabec@redhat.com>
+ *      Šimon Lukašík
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,9 +44,11 @@
 #include <regex.h>
 #endif
 #include <ctype.h>
+#include "oval_agent_api_impl.h"
 #include "oval_results_impl.h"
-#include "oval_collection_impl.h"
-#include "oval_string_map_impl.h"
+#include "adt/oval_collection_impl.h"
+#include "adt/oval_string_map_impl.h"
+#include "collectVarRefs_impl.h"
 #include "public/oval_types.h"
 #include "common/util.h"
 #include "common/debug_priv.h"
@@ -190,7 +193,6 @@ typedef struct oval_result_test {
 	struct oval_collection *bindings;
 	int instance;
 	bool bindings_initialized;
-	bool bindings_clearable;
 } oval_result_test_t;
 
 struct oval_result_test *oval_result_test_new(struct oval_result_system *sys, char *tstid)
@@ -210,7 +212,6 @@ struct oval_result_test *oval_result_test_new(struct oval_result_system *sys, ch
 	test->items = oval_collection_new();
 	test->bindings = oval_collection_new();
 	test->bindings_initialized = false;
-	test->bindings_clearable = false;
 	return test;
 }
 
@@ -218,9 +219,8 @@ struct oval_result_test *oval_result_test_clone
     (struct oval_result_system *new_system, struct oval_result_test *old_test) {
 	__attribute__nonnull__(old_test);
 
-	struct oval_test *oval_test = oval_result_test_get_test(old_test);
-	char *testid = oval_test_get_id(oval_test);
-	struct oval_result_test *new_test = oval_result_test_new(new_system, testid);
+	const char *testid = oval_result_test_get_id(old_test);
+	struct oval_result_test *new_test = oval_result_test_new(new_system, (char *) testid);
 	struct oval_result_item_iterator *old_items = oval_result_test_get_items(old_test);
 	while (oval_result_item_iterator_has_more(old_items)) {
 		struct oval_result_item *old_item = oval_result_item_iterator_next(old_items);
@@ -251,9 +251,11 @@ struct oval_result_test *oval_result_test_clone
 	return new_test;
 }
 
-struct oval_result_test *make_result_test_from_oval_test(struct oval_result_system *sys, struct oval_test *oval_test) {
+struct oval_result_test *make_result_test_from_oval_test(struct oval_result_system *sys, struct oval_test *oval_test, int variable_instance) {
 	char *test_id = oval_test_get_id(oval_test);
-	return oval_result_test_new(sys, test_id);
+	struct oval_result_test *test = oval_result_test_new(sys, test_id);
+	oval_result_test_set_instance(test, variable_instance);
+	return test;
 }
 
 void oval_result_test_free(struct oval_result_test *test)
@@ -262,11 +264,7 @@ void oval_result_test_free(struct oval_result_test *test)
 
 	oval_collection_free_items(test->messages, (oscap_destruct_func) oval_message_free);
 	oval_collection_free_items(test->items, (oscap_destruct_func) oval_result_item_free);
-	if (test->bindings_clearable)
-		oval_collection_free_items(test->bindings, (oscap_destruct_func) oval_variable_binding_free);
-	else {
-		oval_collection_free(test->bindings);	/* avoid leaks - remove collection but not bindings. Bindings are part of syschar model */
-	}
+	oval_collection_free_items(test->bindings, (oscap_destruct_func) oval_variable_binding_free);
 
 	test->system = NULL;
 	test->test = NULL;
@@ -278,30 +276,6 @@ void oval_result_test_free(struct oval_result_test *test)
 	oscap_free(test);
 }
 
-bool oval_result_test_iterator_has_more(struct oval_result_test_iterator
-					*oc_result_test)
-{
-	return oval_collection_iterator_has_more((struct oval_iterator *)
-						 oc_result_test);
-}
-
-struct oval_result_test *oval_result_test_iterator_next(struct
-							oval_result_test_iterator
-							*oc_result_test)
-{
-	return (struct oval_result_test *)
-	    oval_collection_iterator_next((struct oval_iterator *)
-					  oc_result_test);
-}
-
-void oval_result_test_iterator_free(struct
-				    oval_result_test_iterator
-				    *oc_result_test)
-{
-	oval_collection_iterator_free((struct oval_iterator *)
-				      oc_result_test);
-}
-
 struct oval_result_system *oval_result_test_get_system(struct oval_result_test *rtest)
 {
 	__attribute__nonnull__(rtest);
@@ -1514,7 +1488,48 @@ static void _oval_result_test_initialize_bindings(struct oval_result_test *rslt_
 	__attribute__nonnull__(rslt_test);
 
 	struct oval_test *oval_test = oval_result_test_get_test(rslt_test);
+	struct oval_string_map *vm;
+	struct oval_state_iterator *ste_itr;
+	struct oval_iterator *var_itr;
 
+	vm = oval_string_map_new();
+
+	/* Gather bindings pertaining to the referenced states */
+	/* TODO: cache bindings collected for each state */
+	ste_itr = oval_test_get_states(oval_test);
+	while (oval_state_iterator_has_more(ste_itr)) {
+		struct oval_state *ste;
+
+		ste = oval_state_iterator_next(ste_itr);
+		oval_ste_collect_var_refs(ste, vm);
+	}
+	oval_state_iterator_free(ste_itr);
+
+	var_itr = oval_string_map_values(vm);
+	while (oval_collection_iterator_has_more(var_itr)) {
+		struct oval_variable *var;
+		struct oval_value_iterator *val_itr;
+		struct oval_variable_binding *binding;
+
+		var = oval_collection_iterator_next(var_itr);
+		binding = oval_variable_binding_new(var, NULL);
+
+		val_itr = oval_variable_get_values(var);
+		while (oval_value_iterator_has_more(val_itr)) {
+			struct oval_value *val;
+			char *txt;
+
+			val = oval_value_iterator_next(val_itr);
+			txt = oval_value_get_text(val);
+			txt = oscap_strdup(txt);
+			oval_variable_binding_add_value(binding, txt);
+		}
+		oval_value_iterator_free(val_itr);
+		oval_result_test_add_binding(rslt_test, binding);
+	}
+	oval_collection_iterator_free(var_itr);
+
+	/* Gather bindings pertaining to the collected object */
 	struct oval_object *oval_object = oval_test_get_object(oval_test);
 	if (oval_object) {
 		char *object_id = oval_object_get_id(oval_object);
@@ -1525,14 +1540,30 @@ static void _oval_result_test_initialize_bindings(struct oval_result_test *rslt_
 		if(syschar) {
 			struct oval_variable_binding_iterator *bindings = oval_syschar_get_variable_bindings(syschar);
 			while (oval_variable_binding_iterator_has_more(bindings)) {
+				struct oval_variable *var;
+				char *var_id;
 				struct oval_variable_binding *binding = oval_variable_binding_iterator_next(bindings);
-				oval_result_test_add_binding(rslt_test, binding);
+
+				var = oval_variable_binding_get_variable(binding);
+				var_id = oval_variable_get_id(var);
+				/* Don't add bindings that were already
+				 * collected from states. Assumtion is made
+				 * that object's own bindings don't contain
+				 * duplicates.
+				 */
+				if (oval_string_map_get_value(vm, var_id) == NULL) {
+					struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
+					struct oval_variable_binding *binding_copy = oval_variable_binding_clone(binding, definition_model);
+					oval_result_test_add_binding(rslt_test, binding_copy);
+				}
 			}
 			oval_variable_binding_iterator_free(bindings);
 		}
 	}
+
+	oval_string_map_free(vm, NULL);
+
 	rslt_test->bindings_initialized = true;
-	rslt_test->bindings_clearable = false;	//bindings are shared from syschar model.
 }
 
 oval_result_t oval_result_test_eval(struct oval_result_test *rtest)
@@ -1554,7 +1585,7 @@ oval_result_t oval_result_test_eval(struct oval_result_test *rtest)
 			rtest->result = OVAL_RESULT_UNKNOWN;
 	}
 
-        dI("\t%s => %s\n", oval_test_get_id(oval_result_test_get_test(rtest)), oval_result_get_text(rtest->result));
+        dI("\t%s => %s\n", oval_result_test_get_id(rtest), oval_result_get_text(rtest->result));
 
 	return rtest->result;
 }
@@ -1685,13 +1716,13 @@ int oval_result_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_conte
 
 	dmod = context->definition_model;
 	dtst = oval_definition_model_get_new_test(dmod, (char *) test_id);
-	test = oval_result_system_get_new_test(sys, dtst);
+	oval_result_t result = oval_result_parse(reader, "result", 0);
+	int variable_instance = oval_parser_int_attribute(reader, "variable_instance", 1);
+
+	test = oval_result_system_get_new_test(sys, dtst, variable_instance);
 	if (test == NULL)
 		return -1;
-
-	oval_result_t result = oval_result_parse(reader, "result", 0);
 	oval_result_test_set_result(test, result);
-	int variable_instance = oval_parser_int_attribute(reader, "variable_instance", 1);
 	oval_result_test_set_instance(test, variable_instance);
 
 	struct oval_test *ovaltst = oval_result_test_get_test(test);
@@ -1725,7 +1756,6 @@ int oval_result_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_conte
 	return_code = oval_parser_parse_tag(reader, context, (oval_xml_tag_parser) _oval_result_test_parse, args);
 	oval_string_map_free(itemmap, NULL);
 	test->bindings_initialized = true;
-	test->bindings_clearable = true;
 
 	oscap_free(test_id);
 	return return_code;
@@ -1802,3 +1832,10 @@ xmlNode *oval_result_test_to_dom(struct oval_result_test *rslt_test, xmlDocPtr d
 
 	return test_node;
 }
+
+const char *oval_result_test_get_id(const struct oval_result_test *rslt_test)
+{
+	__attribute__nonnull__(rslt_test);
+	struct oval_test *test = oval_result_test_get_test((struct oval_result_test *) rslt_test);
+	return (test == NULL) ? NULL : oval_test_get_id(test);
+}
