@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2013 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
  *
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
+ *      Šimon Lukašík
  */
 
 #ifndef OVAL_RESULTS_IMPL_H_
@@ -34,6 +35,7 @@
 
 #include "oval_definitions_impl.h"
 #include "oval_system_characteristics_impl.h"
+#include "adt/oval_smc_impl.h"
 
 #include "common/util.h"
 
@@ -42,14 +44,17 @@ OSCAP_HIDDEN_START;
 int oval_result_system_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
 xmlNode *oval_result_system_to_dom(struct oval_result_system *, struct oval_results_model *, struct oval_directives_model *, xmlDocPtr, xmlNode *);
 
-struct oval_result_test *oval_result_system_get_new_test(struct oval_result_system *, struct oval_test *);
+struct oval_result_test *oval_result_system_get_new_test(struct oval_result_system *, struct oval_test *, int variable_instance);
 
 int oval_result_definition_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
 struct oval_result_definition *make_result_definition_from_oval_definition(struct oval_result_system *,
-									   struct oval_definition *);
+									   struct oval_definition *,
+									   int variable_instance);
 xmlNode *oval_result_definition_to_dom(struct oval_result_definition *, oval_result_directive_content_t, xmlDocPtr, xmlNode *);
+int oval_result_definition_get_variable_instance_hint(const struct oval_result_definition *definition);
+void oval_result_definition_set_variable_instance_hint(struct oval_result_definition *definition, int new_hint_value);
 
-struct oval_result_test *make_result_test_from_oval_test(struct oval_result_system *system, struct oval_test *oval_test);
+struct oval_result_test *make_result_test_from_oval_test(struct oval_result_system *system, struct oval_test *oval_test, int variable_instance);
 
 int oval_result_test_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
 xmlNode *oval_result_test_to_dom(struct oval_result_test *, xmlDocPtr, xmlNode *);
@@ -58,7 +63,7 @@ xmlNode *oval_result_test_to_dom(struct oval_result_test *, xmlDocPtr, xmlNode *
 int oval_result_item_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
 xmlNode *oval_result_item_to_dom(struct oval_result_item *, xmlDocPtr, xmlNode *);
 
-struct oval_result_criteria_node *make_result_criteria_node_from_oval_criteria_node(struct oval_result_system *, struct oval_criteria_node *);
+struct oval_result_criteria_node *make_result_criteria_node_from_oval_criteria_node(struct oval_result_system *, struct oval_criteria_node *, int variable_instance);
 
 int oval_result_criteria_node_parse(xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
 xmlNode *oval_result_criteria_node_to_dom(struct oval_result_criteria_node *, xmlDocPtr, xmlNode *);
@@ -66,7 +71,8 @@ xmlNode *oval_result_criteria_node_to_dom(struct oval_result_criteria_node *, xm
 oval_result_t oval_result_parse(xmlTextReaderPtr, char *, oval_result_t);
 
 struct oval_result_definition *oval_result_system_get_new_definition(struct oval_result_system *,
-								     struct oval_definition *);
+								     struct oval_definition *,
+								     int variable_instance);
 struct oval_result_test *oval_result_system_get_test(struct oval_result_system *, char *);
 
 struct oresults { 
@@ -85,6 +91,11 @@ oval_result_t ores_get_result_byopr(struct oresults *ores, oval_operator_t op);
 
 void oval_results_model_add_system(struct oval_results_model *, struct oval_result_system *);
 
+struct oval_result_definition_iterator *oval_result_definition_iterator_new(struct oval_smc *mapping);
+struct oval_result_test_iterator *oval_result_test_iterator_new(struct oval_smc *mapping);
+
+const char *oval_result_test_get_id(const struct oval_result_test *test);
+
 OSCAP_HIDDEN_END;
 
 #endif				/* OVAL_RESULTS_IMPL_H_ */
