@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2013 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -37,8 +37,9 @@
 
 #include "oval_agent_api_impl.h"
 #include "oval_system_characteristics_impl.h"
-#include "oval_collection_impl.h"
+#include "adt/oval_collection_impl.h"
 #include "oval_parser_impl.h"
+#include "oval_definitions_impl.h"
 
 #include "common/util.h"
 #include "common/debug_priv.h"
