@@ -60,7 +60,7 @@ void *probe_input_handler(void *arg)
         if (pthread_attr_init(&pth_attr))
                 return (NULL);
 
-        if (pthread_attr_setdetachstate(&pth_attr, PTHREAD_CREATE_DETACHED)) {
+        if (pthread_attr_setdetachstate(&pth_attr, PTHREAD_CREATE_JOINABLE)) {
                 pthread_attr_destroy(&pth_attr);
                 return (NULL);
         }
@@ -110,7 +110,11 @@ void *probe_input_handler(void *arg)
 			SEXP_VALIDATE(oid);
 			probe_out = probe_rcache_sexp_get(probe->rcache, oid);
 
-			if (OSCAP_GSYM(offline_mode) && !OSCAP_GSYM(offline_mode_supported)) {
+			dI("offline_mode=%08x\n", OSCAP_GSYM(offline_mode));
+			dI("offline_supp=%08x\n", OSCAP_GSYM(offline_mode_supported));
+
+			if ((OSCAP_GSYM(offline_mode) != PROBE_OFFLINE_NONE) &&
+			    !(OSCAP_GSYM(offline_mode) & OSCAP_GSYM(offline_mode_supported))) {
 				probe_out = probe_cobj_new(OSCAP_GSYM(offline_mode_cobjflag), NULL, NULL, NULL);
 				if (probe_rcache_sexp_add(probe->rcache, oid, probe_out) != 0) {
 					abort();
@@ -211,6 +215,7 @@ void *probe_input_handler(void *arg)
 				   errno, strerror(errno));
 
 				SEAP_msg_free(seap_request);
+				SEXP_free(probe_in);
 
 				break;
 			}
