@@ -36,6 +36,7 @@
 #include "common/elements.h"
 #include "common/_error.h"
 #include "common/debug_priv.h"
+#include "common/xmlns_priv.h"
 
 struct xccdf_version_info {
 	const char* version; ///< MAJOR.MINOR, for example "1.1" or "1.2"
@@ -71,6 +72,7 @@ const char* xccdf_version_info_get_cpe_version(const struct xccdf_version_info*
 static const struct xccdf_version_info XCCDF_VERSION_MAP[] = {
 	{"1.2", "http://checklists.nist.gov/xccdf/1.2", "2.3"},
 	{"1.1", "http://checklists.nist.gov/xccdf/1.1", "2.0"},
+	{"1.1", "http://open-scap.org/page/Xccdf-1.1-tailoring", "2.0"},
 	{NULL, NULL, NULL}
 };
 
@@ -159,10 +161,7 @@ struct xccdf_element_spec {
 };
 
 #define XCCDF_XMLNS_PREFIX "http://checklists.nist.gov/xccdf"
-/// CPE version 2 Language XML namespace
-#define CPE2L_XMLNS "http://cpe.mitre.org/language/2.0"
-/// CPE version 1 dictionary XML namespace
-#define CPE1D_XMLNS "http://cpe.mitre.org/XMLSchema/cpe/1.0"
+#define XCCDF_OR_TAILORING_EXTENSION_XMLNS "http://open-scap.org/page/Xccdf-1.1-tailoring"
 
 /// Mapping of the element names to the symbolic constants
 static const struct xccdf_element_spec XCCDF_ELEMENT_MAP[] = {
@@ -182,7 +181,7 @@ static const struct xccdf_element_spec XCCDF_ELEMENT_MAP[] = {
 	{XCCDFE_CHOICE, XCCDF_XMLNS_PREFIX, "choice"},
 	{XCCDFE_COMPLEX_CHECK, XCCDF_XMLNS_PREFIX, "complex-check"},
 	{XCCDFE_CONFLICTS, XCCDF_XMLNS_PREFIX, "conflicts"},
-	{XCCDFE_CPE_LIST, CPE1D_XMLNS, "cpe-list"},
+	{XCCDFE_CPE_LIST, XMLNS_CPE1D, "cpe-list"},
 	{XCCDFE_DC_STATUS, XCCDF_XMLNS_PREFIX, "dc-status"},
 	{XCCDFE_DEFAULT, XCCDF_XMLNS_PREFIX, "default"},
 	{XCCDFE_DESCRIPTION, XCCDF_XMLNS_PREFIX, "description"},
@@ -207,7 +206,7 @@ static const struct xccdf_element_spec XCCDF_ELEMENT_MAP[] = {
 	{XCCDFE_PARAM, XCCDF_XMLNS_PREFIX, "param"},
 	{XCCDFE_PLAIN_TEXT, XCCDF_XMLNS_PREFIX, "plain-text"},
 	{XCCDFE_PLATFORM, XCCDF_XMLNS_PREFIX, "platform"},
-	{XCCDFE_CPE2_PLATFORMSPEC, CPE2L_XMLNS, "platform-specification"},
+	{XCCDFE_CPE2_PLATFORMSPEC, XMLNS_CPE2L, "platform-specification"},
 	{XCCDFE_RESULT_PROFILE, XCCDF_XMLNS_PREFIX, "profile"},
 	{XCCDFE_PROFILE_NOTE, XCCDF_XMLNS_PREFIX, "profile-note"},
 	{XCCDFE_QUESTION, XCCDF_XMLNS_PREFIX, "question"},
@@ -227,7 +226,7 @@ static const struct xccdf_element_spec XCCDF_ELEMENT_MAP[] = {
 	{XCCDFE_SOURCE, XCCDF_XMLNS_PREFIX, "source"},
 	{XCCDFE_STATUS, XCCDF_XMLNS_PREFIX, "status"},
 	{XCCDFE_SUB, XCCDF_XMLNS_PREFIX, "sub"},
-	{XCCDFE_TAILORING, XCCDF_XMLNS_PREFIX, "Tailoring"},
+	{XCCDFE_TAILORING, XCCDF_OR_TAILORING_EXTENSION_XMLNS, "Tailoring"},
 	{XCCDFE_TARGET, XCCDF_XMLNS_PREFIX, "target"},
 	{XCCDFE_TARGET_ADDRESS, XCCDF_XMLNS_PREFIX, "target-address"},
 	{XCCDFE_TARGET_FACTS, XCCDF_XMLNS_PREFIX, "target-facts"},
@@ -235,8 +234,9 @@ static const struct xccdf_element_spec XCCDF_ELEMENT_MAP[] = {
 	{XCCDFE_TITLE, XCCDF_XMLNS_PREFIX, "title"},
 	{XCCDFE_UPPER_BOUND, XCCDF_XMLNS_PREFIX, "upper-bound"},
 	{XCCDFE_VALUE_VAL, XCCDF_XMLNS_PREFIX, "value"},
-	{XCCDFE_VERSION, XCCDF_XMLNS_PREFIX, "version"},
+	{XCCDFE_VERSION, XCCDF_OR_TAILORING_EXTENSION_XMLNS, "version"},
 	{XCCDFE_WARNING, XCCDF_XMLNS_PREFIX, "warning"},
+	{XCCDFE_BENCHMARK_REF, XCCDF_XMLNS_PREFIX, "benchmark"},
 	{0, NULL, NULL}
 };
 
@@ -261,8 +261,8 @@ xccdf_element_t xccdf_element_get(xmlTextReaderPtr reader)
 				// namespace exists and matches the table
 				return mapptr->id;
 
-			if (nsuri && strcmp(mapptr->ns, XCCDF_XMLNS_PREFIX) == 0 &&
-			    strncmp(mapptr->ns, nsuri, strlen(mapptr->ns)) == 0)
+			if (nsuri && (strcmp(mapptr->ns, XCCDF_XMLNS_PREFIX) == 0 || strcmp(mapptr->ns, XCCDF_OR_TAILORING_EXTENSION_XMLNS) == 0) &&
+			    strncmp(XCCDF_XMLNS_PREFIX, nsuri, strlen(XCCDF_XMLNS_PREFIX)) == 0)
 				// Namespace exists and XCCDF version-less namespace prefix matches
 				// it. This is done to support multiple XCCDF versions.
 				//
@@ -270,6 +270,11 @@ xccdf_element_t xccdf_element_get(xmlTextReaderPtr reader)
 				// the document. The only thing we are solving here is making sure
 				// the element is some kind of an XCCDF element.
 				return mapptr->id;
+
+			if (nsuri && strcmp(mapptr->ns, XCCDF_OR_TAILORING_EXTENSION_XMLNS) == 0 &&
+			    strcmp(XCCDF_OR_TAILORING_EXTENSION_XMLNS, nsuri) == 0)
+				// In case we are supposed to accept XCCDF tailoring extension
+				return mapptr->id;
 		}
 	}
 
