@@ -37,6 +37,7 @@
 #include "common/util.h"
 #include "common/_error.h"
 #include "DS/public/scap_ds.h"
+#include "DS/ds_common.h"
 #include "XCCDF_POLICY/public/xccdf_policy.h"
 #include "XCCDF_POLICY/xccdf_policy_priv.h"
 #include "item.h"
@@ -65,6 +66,7 @@ struct xccdf_session {
 		struct ds_sds_index *sds_idx;		///< Index of Source DataStream (only applicable for sds).
 		char *user_datastream_id;		///< Datastream id requested by user (only applicable for sds).
 		char *user_component_id;		///< Component id requested by user (only applicable for sds).
+		char *user_benchmark_id;		///< Benchmark id requested by user (only applicable for sds).
 		char *datastream_id;			///< Datastream id used (only applicable for sds).
 		char *component_id;			///< Component id used (only applicable for sds).
 	} ds;
@@ -149,6 +151,7 @@ void xccdf_session_free(struct xccdf_session *session)
 		xccdf_policy_model_free(session->xccdf.policy_model);
 	oscap_free(session->ds.user_datastream_id);
 	oscap_free(session->ds.user_component_id);
+	oscap_free(session->ds.user_benchmark_id);
 	if (session->ds.sds_idx != NULL)
 		ds_sds_index_free(session->ds.sds_idx);
 	if (session->temp_dir != NULL)
@@ -201,6 +204,18 @@ const char *xccdf_session_get_component_id(struct xccdf_session *session)
 	return session->ds.component_id;
 }
 
+void xccdf_session_set_benchmark_id(struct xccdf_session *session, const char *benchmark_id)
+{
+	if (session->ds.user_benchmark_id != NULL)
+		oscap_free(session->ds.user_benchmark_id);
+	session->ds.user_benchmark_id = oscap_strdup(benchmark_id);
+}
+
+const char *xccdf_session_get_benchmark_id(struct xccdf_session *session)
+{
+	return session->ds.user_benchmark_id;
+}
+
 void xccdf_session_set_user_cpe(struct xccdf_session *session, const char *user_cpe)
 {
 	oscap_free(session->user_cpe);
@@ -361,15 +376,37 @@ int xccdf_session_load_xccdf(struct xccdf_session *session)
 		if (session->temp_dir == NULL)
 			goto cleanup;
 
-		if (ds_sds_index_select_checklist(xccdf_session_get_sds_idx(session),
+		// We only use benchmark ID if datastream ID and/or component ID were NOT supplied.
+		if (!session->ds.user_datastream_id && !session->ds.user_component_id && session->ds.user_benchmark_id) {
+			if (ds_sds_index_select_checklist_by_benchmark_id(xccdf_session_get_sds_idx(session),
+				session->ds.user_benchmark_id,
 				(const char **) &(session->ds.datastream_id),
 				(const char **) &(session->ds.component_id)) != 0) {
-			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Failed to locate a datastream with ID matching "
-					"'%s' ID and checklist inside matching '%s' ID.",
-					session->ds.user_datastream_id == NULL ? "<any>" : session->ds.user_datastream_id,
-					session->ds.user_component_id == NULL ? "<any>" : session->ds.user_component_id);
-			goto cleanup;
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Failed to locate a datastream with component-ref "
+					"that points to a component containing Benchmark with ID '%s'.", session->ds.user_benchmark_id);
+				goto cleanup;
+			}
 		}
+		else {
+			if (session->ds.user_benchmark_id) {
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Not using benchmark ID ('%s') for component-ref lookup, "
+					"datastream ID ('%s') and/or component-ref ID ('%s') were supplied, using them instead.",
+					session->ds.user_benchmark_id,
+					session->ds.user_datastream_id,
+					session->ds.user_component_id);
+			}
+
+			if (ds_sds_index_select_checklist(xccdf_session_get_sds_idx(session),
+				(const char **) &(session->ds.datastream_id),
+				(const char **) &(session->ds.component_id)) != 0) {
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Failed to locate a datastream with ID matching "
+						"'%s' ID and checklist inside matching '%s' ID.",
+						session->ds.user_datastream_id == NULL ? "<any>" : session->ds.user_datastream_id,
+						session->ds.user_component_id == NULL ? "<any>" : session->ds.user_component_id);
+				goto cleanup;
+			}
+		}
+
 		if (ds_sds_decompose(session->filename, session->ds.datastream_id, session->ds.component_id, session->temp_dir, XCCDF_XML) != 0) {
 			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Failed to decompose source datastream in '%s'.", session->filename);
 			goto cleanup;
@@ -466,6 +503,7 @@ int xccdf_session_load_cpe(struct xccdf_session *session)
 					session->temp_dir, "dictionaries", NULL, NULL) != 0) {
 				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Can't decompose CPE dictionaries from datastream '%s' "
 						"from file '%s'!\n", session->ds.datastream_id, session->filename);
+				oscap_string_iterator_free(cpe_it);
 				return 1;
 			}
 
@@ -952,69 +990,130 @@ static void _xccdf_session_free_oval_result_files(struct xccdf_session *session)
 	}
 }
 
+static char *_xccdf_session_export_oval_result_file(struct xccdf_session *session, struct oval_agent_session *oval_session)
+{
+	/* get result model and session name */
+	struct oval_results_model *res_model = oval_agent_get_results_model(oval_session);
+	const char* oval_results_directory = NULL;
+	char *name = NULL;
+
+	if (session->export.oval_results == true) {
+		oval_results_directory = ".";
+	}
+	else {
+		if (!session->temp_dir)
+			session->temp_dir = oscap_acquire_temp_dir();
+		if (session->temp_dir == NULL)
+			return NULL;
+
+		oval_results_directory = session->temp_dir;
+	}
+
+	char *escaped_url = NULL;
+	const char *filename = oval_agent_get_filename(oval_session);
+	if (!filename) {
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Given oval_session doesn't have a valid filename. "
+			"Can't deduce destination where we should be exporting it!");
+		return NULL;
+	}
+
+	if (oscap_acquire_url_is_supported(filename) || filename[0] == '/') {
+		// We need escaping if:
+		// - filename is a URL
+		// - filename is an absolute path
+
+		escaped_url = oscap_acquire_url_to_filename(filename);
+		if (escaped_url == NULL)
+			return NULL;
+	}
+	else if (!session->export.oval_results && strchr(filename, '/')) {
+		// We need recursive mkdir_p if:
+		// - filename is not a basename and we are exporting to a temp dir (ARF)
+		// (we are assuming that the dir structure exists if we are exporting to '.')
+
+		char *filename_cpy = oscap_sprintf("%s/%s", oval_results_directory, filename);
+		const char *dirname_ = dirname(filename_cpy);
+		if (ds_common_mkdir_p(dirname_) == -1) {
+			oscap_seterr(OSCAP_EFAMILY_GLIBC, "Failed to create directory '%s' for OVAL result '%s'!.", dirname_, filename);
+			oscap_free(filename_cpy);
+			return NULL;
+		}
+
+		oscap_free(filename_cpy);
+	}
+
+	name = malloc(PATH_MAX * sizeof(char));
+	snprintf(name, PATH_MAX, "%s/%s.result.xml", oval_results_directory, escaped_url != NULL ? escaped_url : filename);
+	if (escaped_url != NULL)
+		free(escaped_url);
+
+	/* export result model to XML */
+	if (oval_results_model_export(res_model, NULL, name) == -1) {
+		free(name);
+		return NULL;
+	}
+
+	/* validate OVAL Results */
+	if (session->validate && session->full_validation) {
+		char *doc_version;
+
+		doc_version = oval_determine_document_schema_version((const char *) name, OSCAP_DOCUMENT_OVAL_RESULTS);
+		if (oscap_validate_document(name, OSCAP_DOCUMENT_OVAL_RESULTS, (const char *) doc_version,
+					_reporter, NULL)) {
+			_validation_failed(name, OSCAP_DOCUMENT_OVAL_RESULTS, doc_version);
+			free(name);
+			free(doc_version);
+			return NULL;
+		}
+		free(doc_version);
+	}
+
+	return name;
+}
+
 int xccdf_session_export_oval(struct xccdf_session *session)
 {
 	_xccdf_session_free_oval_result_files(session);
 
 	if ((session->export.oval_results || session->export.arf_file != NULL) && session->oval.agents) {
+
+		int xccdf_oval_agent_count = xccdf_session_get_oval_agents_count(session);
+		int cpe_oval_agent_count = xccdf_session_get_cpe_oval_agents_count(session);
+
 		/* Export OVAL results */
-		session->oval.result_files = malloc((xccdf_session_get_oval_agents_count(session) + 1) * sizeof(char*));
+		session->oval.result_files = malloc((xccdf_oval_agent_count + cpe_oval_agent_count + 1) * sizeof(char*));
 		session->oval.result_files[0] = NULL;
 		int i;
 		for (i=0; session->oval.agents[i]; i++) {
-			/* get result model and session name*/
-			struct oval_results_model *res_model = oval_agent_get_results_model(session->oval.agents[i]);
-			const char* oval_results_directory = NULL;
-			char *name = NULL;
-
-			if (session->export.oval_results == true) {
-				oval_results_directory = ".";
-			}
-			else {
-				if (!session->temp_dir)
-					session->temp_dir = oscap_acquire_temp_dir();
-				if (session->temp_dir == NULL)
-					return 1;
-
-				oval_results_directory = session->temp_dir;
-			}
-
-			char *escaped_url = NULL;
-			const char *filename = oval_agent_get_filename(session->oval.agents[i]);
-			if (oscap_acquire_url_is_supported(filename)) {
-				escaped_url = oscap_acquire_url_to_filename(filename);
-				if (escaped_url == NULL)
-					return 1;
-			}
-
-			name = malloc(PATH_MAX * sizeof(char));
-			snprintf(name, PATH_MAX, "%s/%s.result.xml", oval_results_directory, escaped_url != NULL ? escaped_url : filename);
-			if (escaped_url != NULL)
-				free(escaped_url);
-
-			/* export result model to XML */
-			if (oval_results_model_export(res_model, NULL, name) == -1) {
-				free(name);
+			char *filename = _xccdf_session_export_oval_result_file(session, session->oval.agents[i]);
+			if (filename == NULL) {
+				_xccdf_session_free_oval_result_files(session);
 				return 1;
 			}
 
-			/* validate OVAL Results */
-			if (session->validate && session->full_validation) {
-				char *doc_version;
-
-				doc_version = oval_determine_document_schema_version((const char *) name, OSCAP_DOCUMENT_OVAL_RESULTS);
-				if (oscap_validate_document(name, OSCAP_DOCUMENT_OVAL_RESULTS, (const char *) doc_version,
-						_reporter, NULL)) {
-					_validation_failed(name, OSCAP_DOCUMENT_OVAL_RESULTS, doc_version);
-					free(name);
-					free(doc_version);
-					return 1;
-				}
-				free(doc_version);
-			}
-			session->oval.result_files[i] = name;
+			session->oval.result_files[i] = filename;
+			session->oval.result_files[i + 1] = NULL;
 		}
-		session->oval.result_files[i] = NULL;
+
+		struct oscap_htable_iterator *cpe_it = xccdf_policy_model_get_cpe_oval_sessions(session->xccdf.policy_model);
+		while (oscap_htable_iterator_has_more(cpe_it)) {
+			const char *key = NULL;
+			struct oval_agent_session *value = NULL;
+			oscap_htable_iterator_next_kv(cpe_it, &key, (void*)&value);
+
+			char *filename = _xccdf_session_export_oval_result_file(session, value);
+			if (filename == NULL) {
+				_xccdf_session_free_oval_result_files(session);
+				oscap_htable_iterator_free(cpe_it);
+				return 1;
+			}
+
+			session->oval.result_files[i] = filename;
+			session->oval.result_files[i + 1] = NULL;
+
+			i++;
+		}
+		oscap_htable_iterator_free(cpe_it);
 	}
 
 	/* Export variables */
@@ -1137,6 +1236,24 @@ unsigned int xccdf_session_get_oval_agents_count(const struct xccdf_session *ses
 	return i;
 }
 
+unsigned int xccdf_session_get_cpe_oval_agents_count(const struct xccdf_session *session)
+{
+	if (session->xccdf.policy_model == NULL) {
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Cannot build xccdf_policy.");
+		return 0;
+	}
+
+	unsigned int i = 0;
+	struct oscap_htable_iterator *it = xccdf_policy_model_get_cpe_oval_sessions(session->xccdf.policy_model);
+	while (oscap_htable_iterator_has_more(it)) {
+		oscap_htable_iterator_next(it);
+		i++;
+	}
+	oscap_htable_iterator_free(it);
+
+	return i;
+}
+
 bool xccdf_session_contains_fail_result(const struct xccdf_session *session)
 {
 	struct xccdf_rule_result_iterator *res_it = xccdf_result_get_rule_results(session->xccdf.result);
