@@ -29,7 +29,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <math.h> /* For isnan() */
 #include <libgen.h>
 
 #include "xccdf_policy_priv.h"
@@ -520,7 +519,7 @@ static struct oscap_list * xccdf_policy_check_get_value_bindings(struct xccdf_po
             if (r_value != NULL) {
                 selector = xccdf_refine_value_get_selector(r_value);
                 /* This refine value changes the value content */
-                if (!isnan(xccdf_refine_value_get_oper(r_value))) {
+                if (xccdf_refine_value_get_oper(r_value) > 0) {
                     binding->operator = xccdf_refine_value_get_oper(r_value);
                 } else binding->operator = xccdf_value_get_oper(value);
 
@@ -1722,6 +1721,11 @@ bool xccdf_policy_model_add_cpe_autodetect(struct xccdf_policy_model *model, con
 	return false;
 }
 
+struct oscap_htable_iterator *xccdf_policy_model_get_cpe_oval_sessions(struct xccdf_policy_model *model)
+{
+	return oscap_htable_iterator_new(model->cpe_oval_sessions);
+}
+
 /**
  * Get ID of XCCDF Profile that belongs to XCCDF Policy
  */
@@ -1933,6 +1937,45 @@ xccdf_policy_add_select(struct xccdf_policy *policy, struct xccdf_select *sel)
 	return _xccdf_policy_add_selector_internal(policy, benchmark, sel, true);
 }
 
+static void _xccdf_policy_add_profile_selectors(struct xccdf_policy* policy, struct xccdf_benchmark *benchmark, struct xccdf_profile *profile) {
+	if (!profile)
+		return;
+
+	const char *parent_profile_id = xccdf_profile_get_extends(profile);
+	struct xccdf_profile *parent_profile = NULL;
+
+	if (parent_profile_id != NULL) {
+		if (strcmp(parent_profile_id, xccdf_profile_get_id(profile)) == 0) {
+			// We are shadowing a profile, we need to get the original profile from
+			// benchmark directly to avoid an endless loop.
+			parent_profile = xccdf_benchmark_get_profile_by_id(benchmark, parent_profile_id);
+		}
+		else {
+			struct xccdf_policy *parent_profile_policy = xccdf_policy_model_get_policy_by_id(xccdf_policy_get_model(policy), parent_profile_id);
+			parent_profile = parent_profile_policy != NULL ? xccdf_policy_get_profile(parent_profile_policy) : NULL;
+		}
+	}
+
+	// Add selectors from parent profile if any, these selectors should be added
+	// before selectors from the "real" policy profile are added so that they
+	// can be overridden.
+	if (parent_profile)
+		_xccdf_policy_add_profile_selectors(policy, benchmark, parent_profile);
+
+	struct xccdf_select_iterator *sel_it = xccdf_profile_get_selects(profile);
+	/* Iterate through selects in profile */
+	while (xccdf_select_iterator_has_more(sel_it)) {
+		struct xccdf_select *sel = xccdf_select_iterator_next(sel_it);
+		if (sel == NULL) {
+			assert(false);
+			continue;
+		}
+
+		struct xccdf_select *clone = xccdf_select_clone(sel);
+		_xccdf_policy_add_selector_internal(policy, benchmark, clone, false);
+	}
+	xccdf_select_iterator_free(sel_it);
+}
 
 /**
  * Constructor for structure XCCDF Policy. Create the structure and resolve all rules
@@ -1944,11 +1987,9 @@ xccdf_policy_add_select(struct xccdf_policy *policy, struct xccdf_select *sel)
 struct xccdf_policy * xccdf_policy_new(struct xccdf_policy_model * model, struct xccdf_profile * profile) {
 
 	struct xccdf_policy             * policy;
-        struct xccdf_benchmark          * benchmark;
-        struct xccdf_item_iterator      * item_it;
-        struct xccdf_item               * item;
-        struct xccdf_select             * sel;
-        struct xccdf_select_iterator    * sel_it = NULL;
+	struct xccdf_benchmark          * benchmark;
+	struct xccdf_item_iterator      * item_it;
+	struct xccdf_item               * item;
 
 	policy = oscap_alloc(sizeof(struct xccdf_policy));
 	if (policy == NULL)
@@ -1958,27 +1999,16 @@ struct xccdf_policy * xccdf_policy_new(struct xccdf_policy_model * model, struct
 	policy->profile = profile;
 	policy->selects = oscap_list_new();
 	policy->values  = oscap_list_new();
-        policy->results = oscap_list_new();
+	policy->results = oscap_list_new();
 
 	policy->selected_internal = oscap_htable_new();
 	policy->selected_final = oscap_htable_new();
-        policy->model = model;
+	policy->model = model;
 
-        benchmark = xccdf_policy_model_get_benchmark(model);
-        if (profile != NULL)
-            sel_it = xccdf_profile_get_selects(profile);
-        /* Iterate through selects in profile */
-        while (xccdf_select_iterator_has_more(sel_it)) {
+	benchmark = xccdf_policy_model_get_benchmark(model);
 
-            sel = xccdf_select_iterator_next(sel_it);
-		if (sel == NULL) {
-			assert(false);
-			continue;
-		}
-		struct xccdf_select *clone = xccdf_select_clone(sel);
-		_xccdf_policy_add_selector_internal(policy, benchmark, clone, false);
-        }
-        xccdf_select_iterator_free(sel_it);
+	if (profile)
+		_xccdf_policy_add_profile_selectors(policy, benchmark, profile);
 
         /* Iterate through items in benchmark and resolve rules */
         item_it = xccdf_benchmark_get_content(benchmark);
@@ -2128,9 +2158,9 @@ bool xccdf_policy_resolve(struct xccdf_policy * policy)
                 
             } else if (xccdf_item_get_type(item) == XCCDF_RULE) {
                 /* Perform all changes in rule */
-                if (!isnan(xccdf_refine_rule_get_role(r_rule)))
+                if (xccdf_refine_rule_get_role(r_rule) > 0)
                     xccdf_rule_set_role((struct xccdf_rule *) item, xccdf_refine_rule_get_role(r_rule));
-                if (!isnan(xccdf_refine_rule_get_severity(r_rule)))
+                if (!xccdf_refine_rule_get_severity(r_rule) > 0)
                     xccdf_rule_set_severity((struct xccdf_rule *) item, xccdf_refine_rule_get_severity(r_rule));
 
             } else {}/* TODO oscap_err ? */;
@@ -2180,7 +2210,11 @@ struct xccdf_result * xccdf_policy_evaluate(struct xccdf_policy * policy)
     const struct xccdf_version_info* version_info = xccdf_benchmark_get_schema_version(benchmark);
     doc_version = xccdf_version_info_get_version(version_info);
 
+#ifdef __USE_GNU
     if (strverscmp("1.2", doc_version) >= 0)
+#else
+    if (strcmp("1.2", doc_version) >= 0)
+#endif
     {
         // we have to enforce a certain type of ids for XCCDF 1.2+
 
@@ -2350,9 +2384,9 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
             if (r_rule == NULL) return item;
 
             new_item = (struct xccdf_item *) xccdf_rule_clone((struct xccdf_rule *) item);
-            if (!isnan(xccdf_refine_rule_get_role(r_rule)))
+            if (xccdf_refine_rule_get_role(r_rule) > 0)
                 xccdf_rule_set_role((struct xccdf_rule *) new_item, xccdf_refine_rule_get_role(r_rule));
-            if (!isnan(xccdf_refine_rule_get_severity(r_rule)))
+            if (xccdf_refine_rule_get_severity(r_rule) > 0)
                 xccdf_rule_set_severity((struct xccdf_rule *) new_item, xccdf_refine_rule_get_severity(r_rule));
             if (xccdf_refine_rule_weight_defined(r_rule))
                 xccdf_rule_set_weight((struct xccdf_rule *) new_item, xccdf_refine_rule_get_weight(r_rule));
